# Synopsis

### What is TReqS

TReqS stands for Tape Request Scheduler. 
It's a software companion to HPSS, that re-organize HPSS staging requests to mainly

- decrease number of tape movements, by grouping files from same tape
- increase staging throughput, by re-ordering files to be staged from same tape
- limit number of allocated drives per drive-model 
- limit HPSS access to granted users (white list)

 
### What is NOT TReqS 

TReqS is **not** a file transfer protocol **nor** a storage system **nor** even a HPSS copy utility. 

It only allows to schedule staging requests in order to make HPSS files available on HPSS disk. 

You then have to use your own transfer protocol to copy the file on your local disk.


### Architecture

TReqS is a client-server application.

Server, written in java, provides a REST web service so you should be able to communicate 
with it with a simple HTTP client like cURL (http://curl.haxx.se/)
The REST API is detailed on doc/rest/ directory.

To avoid all the JSON and HTTP stuff, we provide a TReqS client in Python


### Requirements

- Java (JDK-1.7 recommended)
- JMS server, e.g. Oracle MessageQueue 5.1 
- TReqS server and optionally TReqS client
- HPSS libraries, HPSS granted access, log4cxx libraries

# Installation

### JMS server, e.g. Oracle MessageQueue 5.1 
	
To download the Oracle MessageQueue 5.1 software:
	
```
> curl -O http://download.java.net/mq/open-mq/5.1/latest/openmq5_1-binary-linux.zip
> unzip openmq5_1-binary-linux.zip
```

To start | stop the JMS server
	
```
> nohup MessageQueue5.1/mq/bin/imqbrokerd -bgnd -silent &
> killall imqbrokerd
```

To check the JMS server
	
```
> telnet localhost 7676
	Trying 127.0.0.1...
	Connected to localhost.
	...
	cluster_discovery tcp CLUSTER_DISCOVERY 0
	admin tcp ADMIN 48090
	...
```
		
### TReqS server

Download the TReqS server (unzip the archive) at http://maven.in2p3.fr/fr/in2p3/cc/storage/treqs2/treqs-delivery/
	
Files are organized as follows:
	
```
	<TREQS-HOME>
	  |-bin                : binaries
	  |-etc                : configuration files
	  |-lib                : libraries
	  |-logs               : logs
	  |-data               : database
	  |-web                : web server files
	    |-templates        : HTML templates
	    |-css              : stylesheets
	    |-scripts          : javascripts
	  |-archive            : archive files when using the CSVArchiveModule (see configuration)
	  |-doc                : documentation (admin guide)
	    |-rest             : REST API documentation in HTML format
```

TReqS server is managed by JSW (Java Service Wrapper)
To start | restart | stop the TReqS server
	
```
> <TREQS-HOME>/jsw/treqsd/bin/treqsd start | restart | stop 
```

To check the TReqS server status
	
```
> <TREQS-HOME>/jsw/treqsd/bin/treqsd status	
> curl http://localhost:8080/treqs2/administration/ping
```
	
### TReqS client

Download the TReqS client (unzip the archive) at https://gitlab.in2p3.fr/cc-in2p3-dev/treqs2-client
	

# Quick start

### Starting the TReqS server

It is possible to test TReqS without HPSS instance, but using an embedded HPSS 'simulator' 
(simulator provides metadata of files and simulated staging using sleep()). 
Such files are those defined in FILES4TESTS/ directory.

To do this change HPSS connection in 'etc/treqs2.properties' as follows:

```
treqs.hpss.connector=fr.in2p3.cc.storage.treqs2.hsm.hpss.SimulatedHPSS
```

and then start the TReqS server:

```
> <TREQS-HOME>/bin/treqsd start
```

check it runs properly:
  
```
> <TREQS-HOME>/jsw/treqsd/bin/treqsd status 
> curl http://localhost:8080/treqs2/administration/ping
```


### Issuing requests

#### 1. Submit a staging request

```
> curl -X POST -H 'Content-Type':'application/json' \
    -d '{"file" : {"filename": "/hpss/in2p3.fr/group/ccin2p3/treqs/RUN01/ccwl9159.2773_000100Mb_0001.dat"}}' \
    http://treqs:changeit@localhost:8080/treqs2/staging/request
```

#### 2. Get the request ID
If the submit request succeeds, you will get the ID in the response .e.g. 

 "id":"68a87adc-ba11-4be7-9c3d-e901d1b7609a"

#### 3. Query the status of the request

```
> curl http://treqs:changeit@localhost:8080/treqs2/staging/request/68a87adc-ba11-4be7-9c3d-e901d1b7609a
```

or for a pretty output:

```
> curl -s http://treqs:changeit@localhost:8080/treqs2/staging/request/68a87adc-ba11-4be7-9c3d-e901d1b7609a | python -m json.tool
```

As soon as the request becomes successful, the file is available on HPSS disk.

A successful request has status=ENDED and sub_status=SUCCEEDED.

The file can be:

* status=ENDED, sub_status=ALREADYONDISK if the file was already on disk
* status=ENDED, sub_status=STAGED if the file was staged from HPSS

All these commands can also be done with the TReqS client.

### Monitoring

TReqS provides monitoring web pages.

Point your browser to **http://localhost:8080/treqs2/monitoring/requests** using an authenticated user. 
(possibly default user 'treqs', password 'changeit')

# To go further

To learn more about TReqS, have a look at the admin guide located in <TREQS-HOME>/doc.

Doc is also available at 
**https://gitlab.in2p3.fr/cc-in2p3-dev/treqs2/blob/master/treqs2-delivery/resources/admin-guide.md**

# License

TReqS is a free software 
You can redistribute it and/or modify it under the terms of the GNU Lesser General Public License.

See COPYING and COPYING.LESSER files for details